/******************************************************************************

	COPYRIGHT(C) JONAS 'SORTIE' TERMANSEN 2011.

	This file is part of Sortix.

	Sortix is free software: you can redistribute it and/or modify it under the
	terms of the GNU General Public License as published by the Free Software
	Foundation, either version 3 of the License, or (at your option) any later
	version.

	Sortix is distributed in the hope that it will be useful, but WITHOUT ANY
	WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
	FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
	details.

	You should have received a copy of the GNU General Public License along
	with Sortix. If not, see <http://www.gnu.org/licenses/>.

	memorymanagement.h
	Handles memory for the x86 architecture.

******************************************************************************/

#ifndef SORTIX_MEMORYMANAGEMENT_H
#define SORTIX_MEMORYMANAGEMENT_H

namespace Sortix
{
	struct UnallocPage // Must like this due to assembly.
	{
		size_t Magic;
		void* Next;
		size_t ContinuousPages;
	};

	class Paging
	{
	private:
		UnallocPage* UnallocatedPage; // Must be first in class due to assembly.
		size_t PagesTotal;
		size_t PagesUsed;
		size_t PagesFree;

	public:
		void Init(multiboot_info_t* BootInfo);

	private:
		void Fragmentize();

	public:
		void* Allocate();

	public:
		void Free(void* Page);

	};

	DECLARE_GLOBAL_OBJECT(Paging, Paging);

	class VirtualMemory
	{
	public:
		void Init();

	public:
		void Enable();

	public:
		void Disable();

	};
}

#endif

